(function(){
    /**
     * Save-Return Module
     *
     * Adds a button before the "next" button. On-press a dialog is shown to the user with the current URL plus an info object
     * On returning to this page the info object is picked up in the URL and a return dialog is shown.
     *
     * Placed in
     *  SePlay.saveReturnModule
     *
     * @returns {}
     * init()      initializes
     *
     * @author  Andreas Bernhard <a.bernhard@23go.de>
     * @version 1   2019-06-07
     *
     */


    var MODAL_TEMPLATE = '<div class="modal" tabindex="-1" role="dialog">\n' +
        '  <div class="modal-dialog" role="document">\n' +
        '    <div class="modal-content">\n' +
        '      <div class="modal-header">\n' +
        '        <button type="button" class="close" data-dismiss="modal" aria-label="Close"><span aria-hidden="true">&times;</span></button>\n' +
        '        <h4 class="modal-title"></h4>\n' +
        '      </div>\n' +
        '      <div class="modal-body"></div>\n' +
        '      <div class="modal-footer">\n' +
        '        <button type="button" class="btn btn-primary" data-dismiss="modal">Close</button>\n' +
        '      </div>\n' +
        '    </div><!-- /.modal-content -->\n' +
        '  </div><!-- /.modal-dialog -->\n' +
        '</div><!-- /.modal -->';

    var
        CONST = {
            MODAL_INIT_CLASS: "modal-se-init",
            SAVE_RETURN_PARAMETER_KEY: "d"
        },
        LABELS = {
            modalSaveTitle: 'Save survey state',
            modalSaveBody: 'Save this link to return to the survey later. Paste the link in your browser to continue where you left off. The link will expire in one week.',
            buttonSave: 'Save & leave',
            modalReturnTitle: 'Welcome back.',
            modalReturnBody: 'Your responses from %s have been successfully saved. You can now proceed with the survey'
        },
        UTILS = {
            /**
             * returns object with current query parameters
             */
            queryString: (function(){
                function parseQueryString(){
                    var params = {}, tmp,
                        queryString = location.search ? location.search.substr(1) : false;

                    if(queryString){
                        queryString.split("&").map(function(s){
                            tmp = s.split("=");
                            params[tmp.shift()] = decodeURIComponent(tmp.join("="));
                        });
                    }

                    return params;
                }

                function setQueryString(obj){
                    if(!history || !history.replaceState) return;

                    var qStr = "?";
                    for(var i in obj){
                        qStr += i + "=" + encodeURIComponent(obj[i]);
                    }

                    var newUri = location.protocol + "//" + location.host + location.pathname + qStr + location.hash;
                    history.replaceState(history.state, Date.now(), newUri);
                }

                return {
                    parse: parseQueryString,
                    get: function get(key){
                        return (parseQueryString())[key];
                    },
                    set: function set(key, val){
                        var queryObj = parseQueryString();

                        if(val === undefined){
                            delete(queryObj[key]);
                        }else{
                            queryObj[key] = val;
                        }

                        setQueryString(queryObj);
                    }
                }
            }()),
            /**
             * turns integer value for time in milliseconds into a human readable string, e.g. 3600 => "1h"
             */
            timeToString: function timeToString(timeMs){
                var
                    timeSlots = [
                        ["centurie(s)", 10000],
                        ["year(s)", 100],
                        ["month(s)", 12],
                        ["week(s)", 4],
                        ["day(s)", 7],
                        ["hour(s)", 24],
                        ["minute(s)", 60],
                        ["second(s)", 60],
                        ["millisecond(s)", 1000]
                    ],
                    timeFloat = 1.0 * +timeMs,
                    timeStr, unitStr,
                    timeSlot = timeSlots.pop();

                while(timeFloat > timeSlot[1]){
                    timeFloat = timeFloat / +timeSlot[1];
                    timeSlot = timeSlots.pop();
                }

                timeFloat = Math.round(timeFloat * 10)/10;

                unitStr = timeSlot[0];
                if(timeFloat == 1){
                    unitStr = unitStr.replace("(s)", "");
                }else{
                    unitStr = unitStr.replace("(s)", "s");
                }
                return timeFloat + unitStr;
            },

            sprintf: function sprintf(str, varsArr){
                var val;
                for(var i in varsArr){
                    val = varsArr[i];
                    switch(typeof val){
                        case "string":
                            str = str.replace(/%s/, val);
                            break;
                        case "number":
                            str = str.replace(/%d/, val);
                            break;
                        default:
                            new Error('bad variable format - only string or number supported');
                            break;
                    }
                }

                return str;
            },

            Modal: function(modalId){
                var $modal = $('#' + modalId);

                if(!$modal.length){
                    $modal = $(MODAL_TEMPLATE);
                    $modal.id =  modalId;
                    $('body').append($modal);

                    $modal.on('hide.bs.modal', function(){
                        $('.modal-backdrop.in').remove();
                    })
                }

                return $modal;
            }
        };


    var SePlay = window.SePlay = window.SePlay || {},
        saveReturnModule = SePlay.saveReturnModule;

    if(!saveReturnModule){

        saveReturnModule = (function(){

            var $modalSave,
                $modalReturn,
                $btnSave;

            function initSaveButton(){
                $btnSave = $('#se_save_button');

                if(!$btnSave.length){
                    var $btn_next = $('#se_next_button');

                    $btnSave = $('<button id="se_save_button" type="button" class="btn btn-lg btn-default" style="margin-right:.5em;">' + LABELS.buttonSave + '</button>');
                    $btnSave.on('click', showSaveModal);

                    $btnSave.insertBefore($btn_next);

                }
            }

            function initModals(){
                $modalSave = UTILS.Modal('seSaveModal');

                if(!$modalSave.hasClass(CONST.MODAL_INIT_CLASS)){

                    $modalSave.find('.modal-title').html(LABELS.modalSaveTitle);

                    $modalSave.find('.modal-body')
                        .append(($('<p/>').html(LABELS.modalSaveBody)))
                        .append($('<textarea class="form-control" rows="5" onFocus="this.select()"/>'))
                        .addClass(CONST.MODAL_INIT_CLASS);

                }

                $modalReturn = UTILS.Modal('seReturnModal');

                if(!$modalReturn.hasClass(CONST.MODAL_INIT_CLASS)){

                    $modalReturn.find('.modal-title').html(LABELS.modalReturnTitle);
                    $modalReturn.find('.modal-body')
                        .append(($('<p/>').html(LABELS.modalReturnBody)))
                        .addClass(CONST.MODAL_INIT_CLASS);
                }

            }

            function showSaveModal(){
                var infoParam = btoa(JSON.stringify({
                    date: Date.now()
                }));

                $modalSave.find('textarea').val(location.href + '&' + CONST.SAVE_RETURN_PARAMETER_KEY + '=' + infoParam);
                $modalSave.modal('show');

            }

            function showReturnModal(infoObj){
                var timeObj, timeStr;

                try{
                    timeObj = new Date(infoObj.date);
                    timeStr = UTILS.sprintf("%s, %s", [timeObj.toLocaleDateString(), timeObj.toLocaleTimeString()]);
                }catch(err){
                    // bad info obj?
                }

                if(timeStr){
                    $modalReturn.find('.modal-body p').text(UTILS.sprintf(LABELS.modalReturnBody, [timeStr]));
                }

                $modalReturn.modal('show');

            }

            function checkQuery(){
                var infoStr = UTILS.queryString.get(CONST.SAVE_RETURN_PARAMETER_KEY), infoObj;
                try{
                    infoObj = JSON.parse(atob(infoStr));
                }catch(err){
                    // bad params, bad encoding?
                }

                if(infoStr){
                    showReturnModal(infoObj);
                    UTILS.queryString.set(CONST.SAVE_RETURN_PARAMETER_KEY);
                }
            }

            function init(){
                initModals();
                initSaveButton();
                checkQuery();
            }

            return {
                init: init
            }

        }());

    }

    saveReturnModule.init();

}());